@@ -6,20 +6,17 @@
 #'   score-based statistics are:
 #'   - `"OMG_S"` for the unconditional \eqn{\omega} statistic (Romero et al.,
 #'     2015).
+#'   - `"WOMG_S"` for the unconditional weighted \eqn{\omega} statistic (Trout &
+#'     Gorney, 2025).
 #'   - `"GBT_S"` for the unconditional \eqn{GBT} statistic (van der Linden &
 #'     Sotaridona, 2006).
 #'   - `"M4_S"` for the \eqn{M4} statistic (Maynes, 2014).
 #'
-#'   Options for score and distractor-based statistics are:
-#'   - `"OMG_SD"` for the unconditional \eqn{\omega} statistic (Romero et al.,
-#'     2015).
-#'   - `"GBT_SD"` for the unconditional \eqn{GBT} statistic (van der Linden &
-#'     Sotaridona, 2006).
-#'   - `"M4_SD"` for the \eqn{M4} statistic (Maynes, 2014).
-#'
 #'   Options for response-based statistics are:
 #'   - `"OMG_R"` for the unconditional \eqn{\omega} statistic (Romero et al.,
 #'     2015).
+#'   - `"WOMG_R"` for the unconditional weighted \eqn{\omega} statistic (Trout &
+#'     Gorney, 2025).
 #'   - `"GBT_R"` for the unconditional \eqn{GBT} statistic (van der Linden &
 #'     Sotaridona, 2006).
 #'   - `"M4_R"` for the \eqn{M4} statistic (Maynes, 2014).
@@ -30,18 +27,15 @@
 #'   - `"GBT_ST"` for the unconditional \eqn{GBT} statistic (Gorney & Wollack,
 #'     2024).
 #'
-#'   Options for score, distractor, and response time-based statistics are:
-#'   - `"OMG_SDT"` for the unconditional \eqn{\omega} statistic (Gorney &
-#'     Wollack, 2024).
-#'   - `"GBT_SDT"` for the unconditional \eqn{GBT} statistic (Gorney & Wollack,
-#'     2024).
-#'
 #'   Options for response and response time-based statistics are:
 #'   - `"OMG_RT"` for the unconditional \eqn{\omega} statistic (Gorney &
 #'     Wollack, 2024).
 #'   - `"GBT_RT"` for the unconditional \eqn{GBT} statistic (Gorney & Wollack,
 #'     2024).
 #'
+#' @param x,r,y Matrices of raw data. `x` is for the item scores, `r` the item
+#'   responses, and `y` the item log response times.
+#'   
 #' @inheritParams detect_pm
 #'
 #' @returns A list is returned with the following elements:
@@ -63,6 +57,9 @@
 #' Romero, M., Riascos, Á., & Jara, D. (2015). On the optimality of
 #' answer-copying indices: Theory and practice. *Journal of Educational and
 #' Behavioral Statistics*, *40*(5), 435--453.
+#' 
+#' Trout, N., & Gorney, K. (2025). Weighted answer similarity analysis. *Applied
+#' Psychological Measurement*. Advance online publication.
 #'
 #' van der Linden, W. J., & Sotaridona, L. (2006). Detecting answer copying when
 #' the regular response process follows a known response model. *Journal of
@@ -127,7 +124,7 @@
 #'
 #' # Detect answer similarity
 #' out <- detect_as(
-#'   method = c("OMG_S", "GBT_S", "OMG_ST", "GBT_ST"),
+#'   method = c("OMG_S", "WOMG_S", "GBT_S", "OMG_ST", "GBT_ST"),
 #'   psi = psi,
 #'   x = x,
 #'   y = y
@@ -155,12 +152,12 @@
 #'
 #' # Detect answer similarity
 #' out <- detect_as(
-#'   method = c("OMG_S", "GBT_S"),
+#'   method = c("OMG_S", "WOMG_S", "GBT_S"),
 #'   psi = psi,
 #'   x = x
 #' )
 #'
-#' # Setup for Examples 3 and 4 ------------------------------------------------
+#' # Setup for Example 3 -------------------------------------------------------
 #'
 #' # Settings
 #' set.seed(0)     # seed for reproducibility
@@ -178,50 +175,7 @@
 #'   which(pair[, 1] == p[1] & pair[, 2] == p[2])), 1, 0)
 #' names(ind) <- paste(pair[, 1], pair[, 2], sep = "-")
 #'
-#' # Example 3: Item Scores and Distractors ------------------------------------
-#'
-#' # Generate person parameters for the nested logit model
-#' xi <- MASS::mvrnorm(
-#'   N,
-#'   mu = c(theta = 0.00, eta = 0.00),
-#'   Sigma = matrix(c(1.00, 0.80, 0.80, 1.00), ncol = 2)
-#' )
-#'
-#' # Generate item parameters for the nested logit model
-#' psi <- cbind(
-#'   a = rlnorm(n, meanlog = 0.00, sdlog = 0.25),
-#'   b = rnorm(n, mean = 0.00, sd = 1.00),
-#'   c = runif(n, min = 0.05, max = 0.30),
-#'   lambda1 = rnorm(n, mean = 0.00, sd = 1.00),
-#'   lambda2 = rnorm(n, mean = 0.00, sd = 1.00),
-#'   lambda3 = rnorm(n, mean = 0.00, sd = 1.00),
-#'   zeta1 = rnorm(n, mean = 0.00, sd = 1.00),
-#'   zeta2 = rnorm(n, mean = 0.00, sd = 1.00),
-#'   zeta3 = rnorm(n, mean = 0.00, sd = 1.00)
-#' )
-#'
-#' # Simulate uncontaminated data
-#' dat <- sim(psi, xi)
-#' x <- dat$x
-#' d <- dat$d
-#'
-#' # Modify contaminated data by replacing 40% of the copier scores and
-#' # distractors with source scores and distractors
-#' for (v in 1:length(c)) {
-#'   ci <- sample(1:n, size = n * 0.40)
-#'   x[c[v], ci] <- x[s[v], ci]
-#'   d[c[v], ci] <- d[s[v], ci]
-#' }
-#'
-#' # Detect answer similarity
-#' out <- detect_as(
-#'   method = c("OMG_S", "GBT_S", "OMG_SD", "GBT_SD"),
-#'   psi = psi,
-#'   x = x,
-#'   d = d
-#' )
-#'
-#' # Example 4: Item Responses -------------------------------------------------
+#' # Example 3: Item Responses -------------------------------------------------
 #'
 #' # Generate person parameters for the nominal response model
 #' xi <- cbind(eta = rnorm(N, mean = 0.00, sd = 1.00))
@@ -250,7 +204,7 @@
 #'
 #' # Detect answer similarity
 #' out <- detect_as(
-#'   method = c("OMG_R", "GBT_R"),
+#'   method = c("OMG_R", "WOMG_R", "GBT_R"),
 #'   psi = psi,
 #'   r = r
 #' )
@@ -259,42 +213,37 @@
 detect_as <- function(method,
                       psi,
                       xi = NULL,
-                      x = NULL, d = NULL, r = NULL, y = NULL,
+                      x = NULL, r = NULL, y = NULL,
                       interval = c(-4, 4),
                       alpha = 0.05) {
 
   # Checks
-  if (any(c("S", "SD", "ST", "SDT") %in% extract(method, 2)) &&
+  if (any(c("S", "ST") %in% extract(method, 2)) &&
       any(c("R", "RT") %in% extract(method, 2))) {
     stop("`method` may contain either score-based statistics or ",
          "response-based statistics, but not both.", call. = FALSE)
   }
-  if (any(c("S", "SD", "ST", "SDT") %in% extract(method, 2))) {
+  if (any(c("S", "ST") %in% extract(method, 2))) {
     check_par("x", psi, xi)
-    if (any(c("SD", "SDT") %in% extract(method, 2))) {
-      check_par("d", psi, xi)
-    }
   } else if (any(c("R", "RT") %in% extract(method, 2))) {
     check_par("r", psi, xi)
   }
-  if (any(c("ST", "SDT", "RT") %in% extract(method, 2))) {
+  if (any(c("ST", "RT") %in% extract(method, 2))) {
     check_par("y", psi)
   }
   method <- match.arg(
     arg = unique(method),
-    choices = c("OMG_S", "GBT_S", "M4_S",
-                "OMG_SD", "GBT_SD", "M4_SD",
-                "OMG_R", "GBT_R", "M4_R",
+    choices = c("OMG_S", "WOMG_S", "GBT_S", "M4_S",
+                "OMG_R", "WOMG_R", "GBT_R", "M4_R",
                 "OMG_ST", "GBT_ST",
-                "OMG_SDT", "GBT_SDT",
                 "OMG_RT", "GBT_RT"),
     several.ok = TRUE
   )
-  check_data(x, d, r, y)
+  check_data(x, r, y)
 
   # Setup
-  N <- max(nrow(x), nrow(d), nrow(r), nrow(y))
-  n <- max(ncol(x), ncol(d), ncol(r), ncol(y))
+  N <- max(nrow(x), nrow(r), nrow(y))
+  n <- max(ncol(x), ncol(r), ncol(y))
   pair <- t(combn(N, 2))
   NN <- nrow(pair)
   stat <- pval <- matrix(
@@ -315,29 +264,32 @@ detect_as <- function(method,
 
   # Estimate person parameters
   if (is.null(xi)) {
-    xi <- est(interval, psi, x = x, d = d, r = r, y = y)
+    xi <- est(interval, psi, x = x, r = r, y = y)
   }
 
   # Compute score-based statistics
-  if (any(c("OMG_S", "GBT_S", "M4_S") %in% method)) {
+  if (any(c("OMG_S", "WOMG_S", "GBT_S", "M4_S") %in% method)) {
     m <- count(psi, ignore = "lambda1")
     p_mat <- irt_p(m, psi, xi, ignore = "lambda1")
     for (v in 1:NN) {
-      s <- sum(x[pair[v, 1], ] == x[pair[v, 2], ])
+      s <- as.integer(x[pair[v, 1], ] == x[pair[v, 2], ])
       p <- rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
       if ("OMG_S" %in% method) {
         stat[v, "OMG_S"] <- compute_OMG(s, p)
       }
+      if ("WOMG_S" %in% method) {
+        stat[v, "WOMG_S"] <- compute_WOMG(s, p)
+      }
       if ("GBT_S" %in% method) {
         stat[v, "GBT_S"] <- compute_GBT(s, p)
       }
       if ("M4_S" %in% method) {
         if ("b" %in% colnames(psi)) {
-          s_1 <- sum((x[pair[v, 1], ] == 1) & (x[pair[v, 2], ] == 1))
+          s_1 <- as.integer((x[pair[v, 1], ] == 1) & (x[pair[v, 2], ] == 1))
           p_1 <- p_mat[pair[v, 1], , 2] * p_mat[pair[v, 2], , 2]
         } else {
-          s_1 <- sum((x[pair[v, 1], ] == (m - 1)) &
-                       (x[pair[v, 2], ] == (m - 1)))
+          s_1 <- as.integer((x[pair[v, 1], ] == (m - 1)) &
+                              (x[pair[v, 2], ] == (m - 1)))
           p_1 <- rep(NA, times = n)
           for (i in 1:n) {
             p_1[i] <- p_mat[pair[v, 1], i, m[i]] * p_mat[pair[v, 2], i, m[i]]
@@ -345,59 +297,36 @@ detect_as <- function(method,
         }
         s_0 <- s - s_1
         p_0 <- p - p_1
-        stat[v, "M4_S"] <- compute_M4(c(s_1, s_0), cbind(p_1, p_0, 1 - p))
-      }
-    }
-  }
-
-  # Compute score and distractor-based statistics
-  if (any(c("OMG_SD", "GBT_SD", "M4_SD") %in% method)) {
-    m <- count(psi)
-    p_mat <- irt_p(m, psi, xi)
-    for (v in 1:NN) {
-      s_1 <- sum((x[pair[v, 1], ] == 1) & (x[pair[v, 2], ] == 1))
-      s_0 <- sum(d[pair[v, 1], ] == d[pair[v, 2], ], na.rm = TRUE)
-      s <- s_1 + s_0
-      p <- rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
-      if ("OMG_SD" %in% method) {
-        stat[v, "OMG_SD"] <- compute_OMG(s, p)
-      }
-      if ("GBT_SD" %in% method) {
-        stat[v, "GBT_SD"] <- compute_GBT(s, p)
-      }
-      if ("M4_SD" %in% method) {
-        p_1 <- rep(NA, times = n)
-        for (i in 1:n) {
-          p_1[i] <- p_mat[pair[v, 1], i, m[i]] * p_mat[pair[v, 2], i, m[i]]
-        }
-        p_0 <- p - p_1
-        stat[v, "M4_SD"] <- compute_M4(c(s_1, s_0), cbind(p_1, p_0, 1 - p))
+        stat[v, "M4_S"] <- compute_M4(cbind(s_1, s_0), cbind(p_1, p_0, 1 - p))
       }
     }
   }
 
   # Compute response-based statistics
-  if (any(c("OMG_R", "GBT_R", "M4_R") %in% method)) {
+  if (any(c("OMG_R", "WOMG_R", "GBT_R", "M4_R") %in% method)) {
     m <- count(psi)
     p_mat <- irt_p(m, psi, xi)
     for (v in 1:NN) {
-      s <- sum(r[pair[v, 1], ] == r[pair[v, 2], ])
+      s <- as.integer(r[pair[v, 1], ] == r[pair[v, 2], ])
       p <- rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
       if ("OMG_R" %in% method) {
         stat[v, "OMG_R"] <- compute_OMG(s, p)
       }
+      if ("WOMG_R" %in% method) {
+        stat[v, "WOMG_R"] <- compute_WOMG(s, p)
+      }
       if ("GBT_R" %in% method) {
         stat[v, "GBT_R"] <- compute_GBT(s, p)
       }
       if ("M4_R" %in% method) {
-        s_1 <- sum((r[pair[v, 1], ] == m) & (r[pair[v, 2], ] == m))
+        s_1 <- as.integer((r[pair[v, 1], ] == m) & (r[pair[v, 2], ] == m))
         p_1 <- rep(NA, times = n)
         for (i in 1:n) {
           p_1[i] <- p_mat[pair[v, 1], i, m[i]] * p_mat[pair[v, 2], i, m[i]]
         }
         s_0 <- s - s_1
         p_0 <- p - p_1
-        stat[v, "M4_R"] <- compute_M4(c(s_1, s_0), cbind(p_1, p_0, 1 - p))
+        stat[v, "M4_R"] <- compute_M4(cbind(s_1, s_0), cbind(p_1, p_0, 1 - p))
       }
     }
   }
@@ -408,9 +337,9 @@ detect_as <- function(method,
     p_mat <- irt_p(m, psi, xi, ignore = "lambda1")
     mu <- t(outer(psi[, "beta"], xi[, "tau"], "-"))
     for (v in 1:NN) {
-      s <- sum((x[pair[v, 1], ] == x[pair[v, 2], ]) &
-                 (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
-                 (y[pair[v, 2], ] < mu[pair[v, 2], ]))
+      s <- as.integer((x[pair[v, 1], ] == x[pair[v, 2], ]) &
+                        (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
+                        (y[pair[v, 2], ] < mu[pair[v, 2], ]))
       p <- 0.25 *
         rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
       if ("OMG_ST" %in% method) {
@@ -422,39 +351,15 @@ detect_as <- function(method,
     }
   }
 
-  # Compute score, distractor, and response time-based statistics
-  if (any(c("OMG_SDT", "GBT_SDT") %in% method)) {
-    m <- count(psi)
-    p_mat <- irt_p(m, psi, xi)
-    mu <- t(outer(psi[, "beta"], xi[, "tau"], "-"))
-    for (v in 1:NN) {
-      s_1 <- sum((x[pair[v, 1], ] == 1) & (x[pair[v, 2], ] == 1) &
-                   (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
-                   (y[pair[v, 2], ] < mu[pair[v, 2], ]))
-      s_0 <- sum(d[pair[v, 1], ] == d[pair[v, 2], ] &
-                   (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
-                   (y[pair[v, 2], ] < mu[pair[v, 2], ]), na.rm = TRUE)
-      s <- s_1 + s_0
-      p <- 0.25 *
-        rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
-      if ("OMG_SDT" %in% method) {
-        stat[v, "OMG_SDT"] <- compute_OMG(s, p)
-      }
-      if ("GBT_SDT" %in% method) {
-        stat[v, "GBT_SDT"] <- compute_GBT(s, p)
-      }
-    }
-  }
-
   # Compute response and response time-based statistics
   if (any(c("OMG_RT", "GBT_RT") %in% method)) {
     m <- count(psi)
     p_mat <- irt_p(m, psi, xi)
     mu <- t(outer(psi[, "beta"], xi[, "tau"], "-"))
     for (v in 1:NN) {
-      s <- sum((r[pair[v, 1], ] == r[pair[v, 2], ]) &
-                 (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
-                 (y[pair[v, 2], ] < mu[pair[v, 2], ]))
+      s <- as.integer((r[pair[v, 1], ] == r[pair[v, 2], ]) &
+                        (y[pair[v, 1], ] < mu[pair[v, 1], ]) &
+                        (y[pair[v, 2], ] < mu[pair[v, 2], ]))
       p <- 0.25 *
         rowSums(p_mat[pair[v, 1], , ] * p_mat[pair[v, 2], , ], na.rm = TRUE)
       if ("OMG_RT" %in% method) {
@@ -469,6 +374,8 @@ detect_as <- function(method,
   # Compute p-values
   pval[, grep("OMG", colnames(pval))] <-
     pnorm(stat[, grep("OMG", colnames(stat))], lower.tail = FALSE)
+  pval[, grep("WOMG", colnames(pval))] <-
+    pnorm(stat[, grep("WOMG", colnames(stat))], lower.tail = FALSE)
   pval[, grep("GBT", colnames(pval))] <-
     stat[, grep("GBT", colnames(stat))]
   pval[, grep("M4", colnames(pval))] <-
